# coding: utf-8

"""
    Fiddle Options Platform

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint

import six

from mercury.client.model import Trade  # noqa: F401,E501


class CondorValueException(Exception):
    pass


class BaseCondor(Trade):
    """
    This class must not used directly.
    """

    def __init__(self,
                 symbol=None,
                 open_date=None,
                 direction=None,
                 quantity=None,
                 type=None,  # todo:
                 out_lower_leg=None,  # out_leg_call
                 in_lower_mid_leg=None,   # in_leg_call
                 in_upper_mid_leg=None,    # in_leg_put
                 out_upper_leg=None,   # out_leg_put
                 open_price=None):

        self.__option_legs = [out_lower_leg, in_lower_mid_leg, in_upper_mid_leg, out_upper_leg]

        # todo: create & raise exceptions
        assert all(l.quantity == self.__option_legs[0].quantity for l in self.__option_legs), 'OptionLegs must have the same quantity'

        if direction == 'LONG':
            direction_condition = (out_lower_leg.is_long() and in_lower_mid_leg.is_short()) and \
                                  (in_upper_mid_leg.is_short() and out_upper_leg.is_long())
        else:  # SHORT
            direction_condition = (out_lower_leg.is_short() and in_lower_mid_leg.is_long()) and \
                                  (in_upper_mid_leg.is_long() and out_upper_leg.is_short())

        if not direction_condition:
            raise CondorValueException("Invalid OptionLegs short and long setup for direction: {}".format(direction))

        # Sets the trade type (Condor, IronCondor)
        # refactor: type
        trade_type = self.__class__.__name__
        assert trade_type != 'basecondor', "BaseCondor class must not be used directly"
        trade_type = 'condor'
        if trade_type == 'IronCondor':
            trade_type = 'iron_condor'
        super(BaseCondor, self).__init__(symbol=symbol,
                                         opening_date=open_date,
                                         direction=direction,
                                         quantity=quantity,
                                         option_legs=self.__option_legs,
                                         type=trade_type)


class Condor(BaseCondor):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'symbol': 'str',
        'direction': 'str',
        'quantity': 'int',
        'opening_date': 'date',
        'option_legs': 'list[OptionLeg]',
        'type': 'str'
    }

    attribute_map = {
        'symbol': 'symbol',
        'direction': 'direction',
        'quantity': 'quantity',
        'opening_date': 'openingDate',
        'option_legs': 'optionLegs',
        'type': 'type'
    }

    def __init__(self,
                 symbol=None,
                 open_date=None,
                 type=None,
                 direction=None,
                 quantity=None,
                 out_lower_leg=None,
                 in_lower_mid_leg=None,
                 in_upper_mid_leg=None,
                 out_upper_leg=None,
                 open_price=None):  # noqa: E501
        """Condor - a model defined in Swagger"""  # noqa: E501
        self.discriminator = None

        super(Condor, self).__init__(symbol=symbol,
                                     open_date=open_date,
                                     direction=direction,
                                     quantity=quantity,
                                     type=type,
                                     out_lower_leg=out_lower_leg,
                                     in_lower_mid_leg=in_lower_mid_leg,
                                     in_upper_mid_leg=in_upper_mid_leg,
                                     out_upper_leg=out_upper_leg,
                                     open_price=open_price)

        if (out_lower_leg.strike > in_lower_mid_leg.strike) or \
                (in_upper_mid_leg.strike > out_upper_leg.strike):
            raise CondorValueException('Invalid strikes in condor setup {}'.format(self._option_legs))

        if type:
            if type == 'CALL' and not all(l.is_call() for l in self._option_legs):
                raise CondorValueException('All legs must be CALL type for CALL condor {}'.format(self._option_legs))

            if type == 'PUT' and not all(l.is_put() for l in self._option_legs):
                raise CondorValueException('All legs must be PUT type for PUT condor {}'.format(self._option_legs))

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Condor):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
